import * as i0 from '@angular/core';
import { signal, computed, forwardRef, Component, ChangeDetectionStrategy, ViewChild, Input, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { style, animate, transition, trigger } from '@angular/animations';
import * as i1 from '@angular/common';
import { NgIf, NgForOf, NgStyle } from '@angular/common';

const optionsPanelStyleNotVisible = style({ opacity: 0, transform: 'scaleY(0.75)' });
const optionsPanelStyleVisible = style({ opacity: 1, transform: 'scaleY(1)' });
const animationTiming = '0.15s ease';
function optionsPanelAnimation() {
    return [
        trigger('optionsPanelAnimation', [
            transition(':enter', [
                optionsPanelStyleNotVisible,
                animate(animationTiming, optionsPanelStyleVisible)
            ]),
            transition(':leave', [
                optionsPanelStyleVisible,
                animate(animationTiming, optionsPanelStyleNotVisible)
            ])
        ])
    ];
}
const clearButtonStyleNotVisible = style({ opacity: 0, transform: 'scale(0.25)' });
const clearButtonStyleVisible = style({ opacity: 1, transform: 'scale(1)' });
function clearButtonAnimation() {
    return [
        trigger('clearButtonAnimation', [
            transition(':enter', [
                clearButtonStyleNotVisible,
                animate(animationTiming, clearButtonStyleVisible)
            ]),
            transition(':leave', [
                clearButtonStyleVisible,
                animate(animationTiming, clearButtonStyleNotVisible)
            ])
        ])
    ];
}

class SpSelectComponent {
    /**
     * List of values that can then be selected as an option.<br>
     * An option can either be a primitive or an object, <b>but all options must be of the same type</b>.
     * @param options List of selectable values
     */
    set options(options) {
        this.options$.set(options);
    }
    /**
     * Indicates if a button is displayed to clear the current selection.
     * @param clearable Show or hide the clear button
     */
    set clearable(clearable) {
        this.clearable$.set(clearable);
    }
    /**
     * <b>Only relevant if the options are objects!</b><br><br>
     * Name of the property that should be used to display the option and the selected value.
     * @param property Property name
     */
    set labelProperty(property) {
        this.labelProperty$.set(property);
    }
    /**
     * <b>Only relevant if the options are objects!</b><br><br>
     * Name of the property that should be used to identify an option.<br>
     * If no {@link labelProperty} is set, this property is used to display the option and the selected value instead.
     * @param property Property name
     */
    set keyProperty(property) {
        this.keyProperty$.set(property);
    }
    /**
     * Text that should be displayed if no value is selected.
     * @param placeholder Placeholder text
     */
    set placeholder(placeholder) {
        this.placeholder$.set(placeholder);
    }
    /**
     * Sets the disabled state of this component.
     * @param disabled Disabled state
     */
    set disabled(disabled) {
        this.disabled$.set(disabled);
    }
    constructor(hostElementRef) {
        this.hostElementRef = hostElementRef;
        this.options$ = signal([]);
        this.clearable$ = signal(true);
        this.labelProperty$ = signal(null);
        this.keyProperty$ = signal(null);
        this.placeholder$ = signal('Select');
        this.disabled$ = signal(false);
        this.label$ = computed(() => {
            if (this.controlValue$() != null) {
                if (this.labelProperty$() != null) {
                    return `${this.controlValue$()[this.labelProperty$()]}`;
                }
                if (this.keyProperty$() != null) {
                    return `${this.controlValue$()[this.keyProperty$()]}`;
                }
                return `${this.controlValue$()}`;
            }
            return this.placeholder$();
        });
        this.tabindex$ = computed(() => this.disabled$() ? -1 : 0);
        this.optionRenderLabel$ = computed(() => typeof this.controlValue$() === 'object' && this.labelProperty$() != null);
        this.optionRenderKey$ = computed(() => typeof this.controlValue$() === 'object' && this.labelProperty$() == null && this.keyProperty$() != null);
        this.showOptionPanel$ = signal(false);
        this.optionPanelStyle$ = signal({});
        this.controlValue$ = signal(null);
    }
    canSetValue(value) {
        if (value === undefined || this.options$().length <= 0) {
            return false;
        }
        if (value === null) {
            return true;
        }
        const optionType = typeof this.options$()[0];
        if (optionType !== typeof value) {
            return false;
        }
        if (typeof value !== 'object' && !this.options$().includes(value)) {
            return false;
        }
        if (typeof value === 'object' && this.keyProperty$() != null) {
            return Object.hasOwn(value, this.keyProperty$()) &&
                this.options$()
                    .map(option => option[this.keyProperty$()])
                    .some(optionValue => JSON.stringify(optionValue) === JSON.stringify(value[this.keyProperty$()]));
        }
        return true;
    }
    getOptionByValue(value) {
        if (value == null) {
            return null;
        }
        return this.options$().find(option => {
            if (typeof option !== 'object') {
                return option === value;
            }
            if (this.keyProperty$() != null) {
                return JSON.stringify(option[this.keyProperty$()]) === JSON.stringify(value[this.keyProperty$()]);
            }
            return JSON.stringify(option) === JSON.stringify(value);
        });
    }
    recalculateOptionsPanelStyle() {
        if (this.showOptionPanel$()) {
            return;
        }
        this.optionPanelStyle$.update(() => {
            const modalParent = this.hostElementRef.nativeElement.closest('.sp-modal');
            if (modalParent == null) {
                return {};
            }
            return {
                position: 'fixed',
                width: `${this.hostElementRef.nativeElement.offsetWidth}px`,
                top: `${this.hostElementRef.nativeElement.offsetTop + this.hostElementRef.nativeElement.offsetHeight}px`,
                left: `${this.hostElementRef.nativeElement.offsetLeft}px`,
                bottom: '0px'
            };
        });
    }
    registerOnChange(onChangeFn) {
        this.onChangeFn = onChangeFn;
    }
    registerOnTouched(onTouchedFn) {
        this.onTouchedFn = onTouchedFn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    writeValue(value) {
        if (!this.canSetValue(value)) {
            this.controlValue$.set(null);
            return;
        }
        this.controlValue$.set(this.getOptionByValue(value));
    }
    toggleOptionPanel() {
        if (this.disabled$()) {
            return;
        }
        this.recalculateOptionsPanelStyle();
        this.showOptionPanel$.update(show => !show);
    }
    selectOption(option) {
        this.controlValue$.set(option);
        this.onChangeFn?.(option);
        this.showOptionPanel$.set(false);
        this.focus();
    }
    clearValue(mouseEvent) {
        mouseEvent?.stopPropagation();
        this.selectOption(null);
    }
    isOptionSelected(option) {
        const selectedOption = this.getOptionByValue(this.controlValue$());
        return JSON.stringify(option) === JSON.stringify(selectedOption);
    }
    focus() {
        this.spSelect?.nativeElement?.focus();
    }
    blur(event) {
        this.onTouchedFn?.();
        if (this.optionsPanel != null && this.optionsPanel.nativeElement.contains(event.relatedTarget)) {
            this.focus();
            return;
        }
        this.showOptionPanel$.set(false);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: SpSelectComponent, deps: [{ token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "17.0.8", type: SpSelectComponent, selector: "sp-select", inputs: { options: "options", clearable: "clearable", labelProperty: "labelProperty", keyProperty: "keyProperty", placeholder: "placeholder", disabled: "disabled" }, host: { properties: { "attr.disabled": "this.disabled$()" }, classAttribute: "sp-select" }, providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => SpSelectComponent),
                multi: true
            }
        ], viewQueries: [{ propertyName: "spSelect", first: true, predicate: ["spSelect"], descendants: true, static: true }, { propertyName: "optionsPanel", first: true, predicate: ["optionsPanel"], descendants: true }], ngImport: i0, template: "<div #spSelect class=\"select-input\" [tabindex]=\"tabindex$()\" (blur)=\"blur($event)\" (click)=\"toggleOptionPanel()\">\r\n    <div class=\"select-label\" [class.placeholder]=\"controlValue$() == null\">\r\n        <span>{{ label$() }}</span>\r\n    </div>\r\n    <div class=\"select-actions\">\r\n        @if (!disabled$() && clearable$() && controlValue$() != null) {\r\n            <div class=\"button-clear-selection\" (click)=\"clearValue($event)\" @clearButtonAnimation>\r\n                <svg width=\"100%\" height=\"100%\" viewBox=\"0 0 26 26\" style=\"fill-rule:evenodd;clip-rule:evenodd;stroke-linejoin:round;stroke-miterlimit:2;\">\r\n                    <path d=\"M24.042 1.414a2 2 0 0 0-2.828 0l-19.8 19.8a2 2 0 0 0 2.829 2.828l19.799-19.8a2 2 0 0 0 0-2.828z\"/>\r\n                    <path d=\"M1.414 1.414a2 2 0 0 0 0 2.828l19.8 19.8a2 2 0 0 0 2.828-2.829L4.24 1.413a1.998 1.998 0 0 0-2.825 0l-.002.001z\"/>\r\n                </svg>\r\n            </div>\r\n        }\r\n        <div class=\"select-trigger\">\r\n            <svg width=\"100%\" height=\"100%\" viewBox=\"0 0 26 14\" style=\"fill-rule:evenodd;clip-rule:evenodd;stroke-linejoin:round;stroke-miterlimit:2;\">\r\n                <path d=\"M.4 3.2A1.999 1.999 0 0 1 2 0h16a2 2 0 0 1 1.6 3.2l-7.2 9.6a3 3 0 0 1-4.8 0L.4 3.2z\"/>\r\n            </svg>\r\n        </div>\r\n    </div>\r\n</div>\r\n\r\n@if (showOptionPanel$()) {\r\n    <div #optionsPanel [ngStyle]=\"optionPanelStyle$()\" class=\"select-options-panel\" @optionsPanelAnimation >\r\n        @for (option of options$(); track $index) {\r\n            <div class=\"select-option\" [class.selected]=\"isOptionSelected(option)\" (click)=\"selectOption(option)\" tabindex=\"-1\">\r\n                @if (optionRenderLabel$()) {\r\n                    <span>{{ option[labelProperty$()!] }}</span>\r\n                } @else if (!optionRenderLabel$() && optionRenderKey$()) {\r\n                    <span>{{ option[keyProperty$()!] }}</span>\r\n                } @else {\r\n                    <span>{{ option }}</span>\r\n                }\r\n            </div>\r\n        } @empty {\r\n            <div class=\"select-option\">\r\n                <span>No Options</span>\r\n            </div>\r\n        }\r\n    </div>\r\n}\r\n\r\n", styles: [""], dependencies: [{ kind: "directive", type: i1.NgStyle, selector: "[ngStyle]", inputs: ["ngStyle"] }], animations: [
            optionsPanelAnimation(),
            clearButtonAnimation()
        ], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: SpSelectComponent, decorators: [{
            type: Component,
            args: [{ selector: 'sp-select', changeDetection: ChangeDetectionStrategy.OnPush, host: {
                        'class': 'sp-select',
                        '[attr.disabled]': 'this.disabled$()'
                    }, animations: [
                        optionsPanelAnimation(),
                        clearButtonAnimation()
                    ], providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(() => SpSelectComponent),
                            multi: true
                        }
                    ], template: "<div #spSelect class=\"select-input\" [tabindex]=\"tabindex$()\" (blur)=\"blur($event)\" (click)=\"toggleOptionPanel()\">\r\n    <div class=\"select-label\" [class.placeholder]=\"controlValue$() == null\">\r\n        <span>{{ label$() }}</span>\r\n    </div>\r\n    <div class=\"select-actions\">\r\n        @if (!disabled$() && clearable$() && controlValue$() != null) {\r\n            <div class=\"button-clear-selection\" (click)=\"clearValue($event)\" @clearButtonAnimation>\r\n                <svg width=\"100%\" height=\"100%\" viewBox=\"0 0 26 26\" style=\"fill-rule:evenodd;clip-rule:evenodd;stroke-linejoin:round;stroke-miterlimit:2;\">\r\n                    <path d=\"M24.042 1.414a2 2 0 0 0-2.828 0l-19.8 19.8a2 2 0 0 0 2.829 2.828l19.799-19.8a2 2 0 0 0 0-2.828z\"/>\r\n                    <path d=\"M1.414 1.414a2 2 0 0 0 0 2.828l19.8 19.8a2 2 0 0 0 2.828-2.829L4.24 1.413a1.998 1.998 0 0 0-2.825 0l-.002.001z\"/>\r\n                </svg>\r\n            </div>\r\n        }\r\n        <div class=\"select-trigger\">\r\n            <svg width=\"100%\" height=\"100%\" viewBox=\"0 0 26 14\" style=\"fill-rule:evenodd;clip-rule:evenodd;stroke-linejoin:round;stroke-miterlimit:2;\">\r\n                <path d=\"M.4 3.2A1.999 1.999 0 0 1 2 0h16a2 2 0 0 1 1.6 3.2l-7.2 9.6a3 3 0 0 1-4.8 0L.4 3.2z\"/>\r\n            </svg>\r\n        </div>\r\n    </div>\r\n</div>\r\n\r\n@if (showOptionPanel$()) {\r\n    <div #optionsPanel [ngStyle]=\"optionPanelStyle$()\" class=\"select-options-panel\" @optionsPanelAnimation >\r\n        @for (option of options$(); track $index) {\r\n            <div class=\"select-option\" [class.selected]=\"isOptionSelected(option)\" (click)=\"selectOption(option)\" tabindex=\"-1\">\r\n                @if (optionRenderLabel$()) {\r\n                    <span>{{ option[labelProperty$()!] }}</span>\r\n                } @else if (!optionRenderLabel$() && optionRenderKey$()) {\r\n                    <span>{{ option[keyProperty$()!] }}</span>\r\n                } @else {\r\n                    <span>{{ option }}</span>\r\n                }\r\n            </div>\r\n        } @empty {\r\n            <div class=\"select-option\">\r\n                <span>No Options</span>\r\n            </div>\r\n        }\r\n    </div>\r\n}\r\n\r\n" }]
        }], ctorParameters: () => [{ type: i0.ElementRef }], propDecorators: { spSelect: [{
                type: ViewChild,
                args: ['spSelect', { static: true }]
            }], optionsPanel: [{
                type: ViewChild,
                args: ['optionsPanel']
            }], options: [{
                type: Input
            }], clearable: [{
                type: Input
            }], labelProperty: [{
                type: Input
            }], keyProperty: [{
                type: Input
            }], placeholder: [{
                type: Input
            }], disabled: [{
                type: Input
            }] } });

class SelectModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: SelectModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "17.0.8", ngImport: i0, type: SelectModule, declarations: [SpSelectComponent], imports: [NgIf,
            NgForOf,
            NgStyle], exports: [SpSelectComponent] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: SelectModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: SelectModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        SpSelectComponent
                    ],
                    imports: [
                        NgIf,
                        NgForOf,
                        NgStyle
                    ],
                    exports: [
                        SpSelectComponent
                    ]
                }]
        }] });

/*
 * Public API Surface of select
 */

/**
 * Generated bundle index. Do not edit.
 */

export { SelectModule, SpSelectComponent };
//# sourceMappingURL=spicysources-select.mjs.map
